<?php
Namespace App\Model;

/**
 * Provides base functions for Homegear rooms
 *
 */
class HomegearRoom
{
    const componentRoomSort = 'sortRooms';
    const componentRoomGridSort = 'sortRoomsGrid';
    const componentDeviceFrontendHidden = 'deviceFrontendHidden';
    public $hg;
    public $defaultLanguage;

    public function __construct($params)
    {
        $this->hg = new \Homegear\Homegear;
        $this->defaultLanguage = $params->get('default_locale');
    }

    /**
     * Load all known rooms
     * 
     * @return array()
     */
    public function loadAll($stories = false)
    {
        // load all rooms, return if there are none
        try {
            $rooms = $this->hg->getRooms();
        } catch(\Homegear\HomegearException $e) {
            return;
        }
        if (is_array($rooms) && count($rooms) == 0) { return; }
        // DEPRECATED: sorting is done by homegear directly!
        $rooms = $this->_sortRooms($rooms);
        if ($stories !== false) {
            $rooms = $this->_getStoryAndBuildingForRoom($rooms,$stories);
        }
        return $rooms;
    }

    /**
     * Load a specific room w/ devices
     * 
     * @return array()
     */
    public function loadById($id = null)
    {
        $allRooms = $this->loadAll();
        if (!is_array($allRooms)) {
            // no rooms?
            return;
        }
        if ($id === null || $id == '') { 
            // no id given, load first room
            if (is_array($allRooms) && count($allRooms) > 0) {
                $id = $allRooms[0]['ID'];
            }
        }
        if ($id !== null) {
            // get the room metadata out of the room-array
            $result = $allRooms[array_search($id, array_column($allRooms,'ID'))];
            // load sorting
            $sort = json_decode($this->hg->getData($this::componentRoomGridSort,"$id"),true);
            // load devices, channels, variables and systemVariables
            $devices = $this->hg->getDevicesInRoom(intval($id));
            $channels = $this->hg->getChannelsInRoom(intval($id));
            $variables = $this->hg->getVariablesInRoom(intval($id));
            $systemVariables = $this->hg->getSystemVariablesInRoom(intval($id));
            $hiddenDevices = $this->hg->getData($this::componentDeviceFrontendHidden);
            foreach ($devices as $key => $device) {
                // check if device is visible
                if (is_array($hiddenDevices) && array_key_exists($device, $hiddenDevices)) {
                    unset($devices[$key]);
                } else {
                    $result['devices'][] = ['ID' => $device, 'channel' => '-1', 'variable' => ''];
                }
            }
            #foreach ($devices as $device) {
            #    $result['devices'][] = ['ID' => $device, 'channel' => '-1', 'variable' => ''];
            #}
            foreach ($channels as $device => $data) {
                foreach ($data as $key => $channel) {
                    if ($channel == -1) { continue; }
                    $result['devices'][] = ['ID' => $device, 'channel' => $channel, 'variable' => ''];
                }
            }
            foreach ($variables as $device => $value) {
                foreach ($value as $channel => $data) {
                    if ($channel == -1) { continue; }
                    foreach ($data as $key => $variable) {
                        $result['devices'][] = ['ID' => $device, 'channel' => $channel, 'variable' => $variable];
                    }
                }
            }
            foreach ($systemVariables as $device) {
                $result['devices'][] = ['ID' => 'systemVariable', 'channel' => '', 'variable' => $device, 'name' => $device, 'url' => base64_encode($device)];
            }
            if (array_key_exists('devices', $result)) {
                foreach ($result['devices'] as $k => $v) {
                    $key = 0;
                    if (is_array($sort) && count($sort) > 0) {
                        $key = array_search($v['ID']."-".$v['channel']."-".$v['variable'], $sort);
                    }
                    $v['sort'] = (isset($key) ? $key : 0);
                    $result['devices'][$k] = $v;
                }
                $result['devices'] = ((is_array($result['devices']) && count($result['devices']) > 0) ? array_values($result['devices']) : null);
            }
            return $result;
        }
    }

    public function loadForStories($stories = false)
    {
        if ($stories === false) { return; }
        $allRooms = $this->loadAll();
        $myRooms = array();
        foreach ($stories as $key => $story) {
            $myRooms[$key] = $story;
            $myRooms[$key]['isStory'] = true;
            if (count($story['ROOMS']) > 0) {
                foreach ($story['ROOMS'] as $rkey => $room) {
                    $current = array_search($room, array_column($allRooms,'ID'));
                    $myRooms[$key]['ROOMS'][$rkey] = $allRooms[$current];
                }
            }
        }
        return $myRooms;
    }

    private function _getStoryAndBuildingForRoom($rooms = false, $stories = false)
    {
        if ($rooms === false || $stories === false) { return; }
        if (is_array($stories)) {
            foreach ($stories as $s) {
                if (is_array($s['ROOMS']) && count($s['ROOMS']) > 0) {
                    foreach ($s['ROOMS'] as $r) {
                        $key = array_search($r, array_column($rooms, 'ID'));
                        if (isset($key) && $key !== NULL) {
                            $rooms[$key]['STORY'] = ['ID' => $s['ID'], 'NAME' => $s['TRANSLATIONS']];
                            if (array_key_exists('BUILDING',$s)) {
                                $rooms[$key]['BUILDING'] = ['ID' => $s['BUILDING']['ID'], 'NAME' => $s['BUILDING']['NAME']];
                            }
                        }
                    }
                }
            }
        }
        return $rooms;
    }

    /**
     * Sort rooms
     * 
     * DEPRECATED: sorting is done by homegear directly!
     * This is only needed once to convert from db-style to homegear-style
     * @return array()
     */
    private function _sortRooms($rooms)
    {
        // load sorting data
        $sort = $this->hg->getData($this::componentRoomSort);
        if ($sort !== null) {
            foreach ($rooms as $room) {
                if (array_key_exists($room['ID'],$sort)) {
                    $metadata = $this->hg->getRoomMetadata($room['ID']);
                    if(!is_array($metadata)) $metadata = array();
                    $metadata['position'] = intval($sort[$room['ID']]);
                    $this->hg->setRoomMetadata($room['ID'], $metadata);
                }
            }
            $rooms = $this->sortDataByComponent($rooms,$sort);
            // delete sorting data since it's not needed anymore
            $this->hg->deleteData($this::componentRoomSort);
        }
        return $rooms;
    }

    /**
     * DEPRECATED: sorting is done by homegear directly!
     * 
     */
    private function sortDataByComponent($dataToSort,$component)
    {
        foreach ($dataToSort as $k => $v) {
            $dataToSort[$k]['sort'] = (isset($component[$v['ID']]) ? $component[$v['ID']] : 0);
        }
        usort($dataToSort, function($a, $b) {
            $retval = $a['sort'] <=> $b['sort'];
            if ($retval == 0) { $retval = $a['ID'] <=> $b['ID']; }
            return $retval;
        });
        return $dataToSort;
    }

}