<?php
Namespace App\Model;

/**
 * Provides base functions for Homegear service messages
 *
 */
class HomegearServiceMessage
{
    const componentServiceMessageIgnores = 'serviceMessageIgnores';
    public $hg;

    public function __construct()
    {
        $this->hg = new \Homegear\Homegear;
    }

    /**
     * Returns the service messages.
     *
     * @return array()
     */
    public function list()
    {
        $messages = $this->hg->getServiceMessages(true);
        $updates = $this->hg->getData("Updates");
        $ignores = $this->getIgnores();
        $families = $this->hg->listFamilies();
        if (is_array($updates) && (array_key_exists('UpdateAvailable_Homegear',$updates) || array_key_exists('UpdateAvailable_System',$updates))) {
            array_unshift($messages, ['TYPE' => 99, 'TIMESTAMP' => $updates['lastUpdateCheck'], 'MESSAGE' => 'UPDATES_AVAILABLE']);
        }
        if (count($messages) > 0) {
            foreach ($messages as $k => $message) {
                if ($message['TYPE'] == 1) {
                    // don't show family-messages id family is not loaded
                    if (!array_search($message['FAMILY_ID'], array_column($families, 'ID'))) {
                        unset($messages[$k]);
                        continue;
                    }
                    // (hopefully) temporary hack for ccu: the serial is case-sensitive in db, but we only get uppercase back :/
                    if ($message['FAMILY_ID'] == 24) {
                        $messages[$k]['DATA']['SERIALNUMBER'] = strtoupper($message['DATA']['SERIALNUMBER']);
                        list($messages[$k]['MESSAGE'],$tmp) = explode('.',$message['MESSAGE']);
                    }
                }
                if ($message['TYPE'] == 2) {
                    // check if device is ignored
                    if (array_key_exists('MESSAGE',$message) && $message['MESSAGE'] == 'STICKY_UNREACH' && is_array($ignores) && array_key_exists($message['PEER_ID'], $ignores)) {
                        unset($messages[$k]);
                        continue;
                    }
                    $deviceInfo = $this->hg->getDeviceDescription(intval($message['PEER_ID']), -1, ["ADDRESS", "NAME"]);
                    $messages[$k]['deviceName'] = ($deviceInfo['NAME'] != '' ? $deviceInfo['NAME'] : $deviceInfo['ADDRESS']);
                }
                if (array_key_exists('NAME',$message)) {
                    $messages[$k]['MESSAGE'] = $message['NAME'];
                }
                $messages[$k]['TIMESTAMP'] = ($message['TIMESTAMP'] == 0 ? '' : date("d.m.Y H:i:s", $message['TIMESTAMP']));
            }
            return $messages;
        }
    }

    /**
     * Delete a service message
     * 
     * @return none
     */
    public function delete($id = null, $dp = null)
    {
        if ($id === null || $dp === null) { return; }
        $device = $this->hg->getDeviceDescription(intval($id),0);
        if (array_key_exists('FAMILY',$device) && $device['FAMILY'] == 24) {
            // CCU servicemessages have to be cleared by script-api!
            $script = "object alarmObject;alarmObject = dom.GetObject(\"AL-%%DEVICE_ID_TO_REPLACE%%:0.STICKY_UNREACH\");if (alarmObject){if (alarmObject.Value()){alarmObject.AlReceipt();}}";
            $gateways = $this->hg->listInterfaces(24);
            $interface = $this->hg->getDeviceInfo(intval($id), array('INTERFACE'));
            $found = array_search($interface['INTERFACE'], array_column($gateways, 'ID'));
            if ($found !== False) {
                // gateway found!
                $script = str_replace("%%DEVICE_ID_TO_REPLACE%%", $device['PARENT'], $script);
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10); // connect timeout
                curl_setopt($ch, CURLOPT_TIMEOUT, 300); // connect timeout
                curl_setopt($ch, CURLOPT_URL, $gateways[$found]['IP_ADDRESS'].':8181/tclrega.exe');
                curl_setopt($ch, CURLOPT_HEADER, 0);
                curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS,$script);
                $result = curl_exec($ch);
                $result = utf8_encode($result);
                $error_string = curl_error($ch);
                $error = curl_errno($ch);
                curl_close($ch);
            }
        }
        $this->hg->setValue(intval($id), 0, "$dp", false);
    }

    /**
     * Delete all service messages
     * 
     * @return none
     */
    public function deleteAll()
    {
        $messages = $this->hg->getServiceMessages(true);
        if (count($messages) > 0) {
            foreach ($messages as $k => $message) {
                if ($message['MESSAGE'] == "STICKY_UNREACH") {
                    $this->delete($message['PEER_ID'],$message['MESSAGE']);
                }
            }
        }
    }

    /**
     * Get all ignored service messages devices
     * 
     * @return none
     */
    public function getIgnores()
    {
        $ignores = $this->hg->getData($this::componentServiceMessageIgnores);
        if (is_array($ignores)) {
            foreach ($ignores as $key => $value) {
                // use try / catch since it could be possible, that a module could not be loaded
                try {
                    $ignores[$key] = $this->hg->getName($key);
                } catch(\Homegear\HomegearException $e) { }
            }
            return $ignores;
        }
    }

    /**
     * Set ignore service messages for device
     * 
     * @return none
     */

    public function setIgnore($id = null)
    {
        if ($id === null) { return; }
        $this->hg->setData($this::componentServiceMessageIgnores,$id,true);
    }

    /**
     * Delete ignore service messages for device
     * 
     * @return none
     */
    public function deleteIgnore($id = null)
    {
        if ($id === null) { return; }
        $this->hg->deleteData($this::componentServiceMessageIgnores,$id);
    }
}